
// Check position of rotary switches and set IP stuff, run DHCP code
// as needed, and return true if we're supposed to boot Linux or false
// if we're not.
//
// Assumes that PIO controllers are all setup and FPGA is programmed.

//#define DEBUG
#include <common.h>

#ifdef DEBUG
/* #include <assert.h> */
#define assert(x) do { if (!(x)) { printf("assert fail in %s() at line %d\n",__FUNCTION__,__LINE__); while(1) udelay(1000); } } while(0)
#define xassert(x) /* noop */
#define Log() printf("%s:%d\n",__FUNCTION__,__LINE__)
#else
#define assert(x) ((void)0)
#define Log() /* noop */
#endif


#define Bit(n)  (1<<(n))
#define msleep(ms) udelay((ms)*1000)

static inline int dhcpEnabled(void)
{
  char *e = getenv("ipmode");
  return e && !strcmp(e,"dhcp");
}

char *ip_as_str(IPaddr_t ip)
{
  static char buf[18];
  ip_to_string(ip,buf);
  return buf;
}

static int do_checkswitches(cmd_tbl_t * cmdtp, int flag, int argc, char *argv[])
{
  uint16_t switchState = *((uint16_t*)0x10000c40);

  // flip digits around
  switchState = ((switchState & 0xf) << 8) | (switchState & 0x0f0) | ((switchState & 0xf00) >> 8);

  printf("Switch state: %03x\n",switchState);

  NetOurIP = getenv_IPaddr("ipaddr");

  if (NetOurIP == 0 || switchState == 0x999 || switchState == 0x801 || switchState == 0x888)
    {
      printf("Loading default IP config\n");
      NetOurIP         = string_to_ip(CONFIG_DEFAULT_IPADDR);
      NetOurSubnetMask = string_to_ip(CONFIG_DEFAULT_NETMASK);
      NetOurGatewayIP  = string_to_ip(CONFIG_DEFAULT_GATEWAYIP);
    }
  else
    {
      printf("Loading flash IP config\n");
      NetOurIP         = getenv_IPaddr("ipaddr");
      NetOurSubnetMask = getenv_IPaddr("netmask");
      NetOurGatewayIP  = getenv_IPaddr("gatewayip");
    }
  
  printf("%s ", ip_as_str(NetOurIP));
  printf("%s ", ip_as_str(NetOurSubnetMask));
  printf("%s\n",ip_as_str(NetOurGatewayIP));

  if (switchState >= 0x001 && switchState <= 0x254)
    {
      NetOurIP &= 0x00ffffff;
      NetOurIP |= (((switchState>>8)&0xf)*100 + ((switchState>>4)&0xf)*10 + (switchState&0xf))<<24;
      printf("Override IP addr LSB with switch position: %s\n", ip_as_str(NetOurIP));
    }
  else if (switchState == 0x998 || switchState == 0x802 || dhcpEnabled())
    {
      // use DHCP
      printf("Using DHCP\n");
      NetOurIP = NetOurSubnetMask = NetOurGatewayIP = 0;
      run_command("dhcp",0);
    }
  else if (switchState == 0x888)
    {
      // reset to factory defaults
      extern int do_spi_flash_sysinfo(int verbose);
      extern void set_default_env(void);
      printf("Resetting environment to factory defaults\n");
      set_default_env();
      do_spi_flash_sysinfo(1);
      saveenv();
      saveenv();
    }

  NetInitAlreadyDone = 1;

  // supposed to stay in UBoot rather than boot Linux?
  if (switchState == 0x800 || switchState == 0x801 || switchState == 0x802)
    {
      printf("Not booting kernel -- staying in U-Boot\n");
      return 1;
    }
  
  return 0;
}

U_BOOT_CMD(checkswitches, 1, 0, do_checkswitches, "check rotary switch settings", "");
